Package: HospitalNetwork
Type: Package
Title: Building Networks of Hospitals Through Patients Transfers
Version: 0.9.4
Authors@R: c(
  person("Pascal", "Crépey", email = "pascal.crepey@ehesp.fr", role = c("aut", "cre", "cph")),
  person("Tjibbe", "Donker", email = "tjibbe.donker@uniklinik-freiburg.de", role = "aut"),
  person("Clément", "Massonnaud", email = "clement.massonnaud@gmail.com", role = "aut"),
  person("Michael", "Lydeamore", email="michael.lydeamore@monash.edu", role=c("aut"))
  )
Description: Set of tools to help interested researchers to build hospital networks 
  from data on hospitalized patients transferred between hospitals. Methods provided 
  have been used in Donker T, Wallinga J, Grundmann H. (2010) <doi:10.1371/journal.pcbi.1000715>, 
  and Nekkab N, Crépey P, Astagneau P, Opatowski L, Temime L. (2020) <doi:10.1038/s41598-020-71212-6>.
URL: https://pascalcrepey.github.io/HospitalNetwork/
BugReports: https://github.com/PascalCrepey/HospitalNetwork/issues
License: GPL-3
Encoding: UTF-8
LazyLoad: true
Imports: checkmate, igraph, lubridate, R6, ggplot2, ggraph
Depends: data.table
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, testthat (>= 2.1.0), shiny, shinyWidgets,
        shinydashboard, DT, shinyalert, shinyjs, vdiffr, pander, glue,
        golem, htmltools
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2024-12-22 01:15:55 UTC; pascalcrepey
Author: Pascal Crépey [aut, cre, cph],
  Tjibbe Donker [aut],
  Clément Massonnaud [aut],
  Michael Lydeamore [aut]
Maintainer: Pascal Crépey <pascal.crepey@ehesp.fr>
Repository: CRAN
Date/Publication: 2024-12-22 04:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:43:44 UTC; unix
