# Generated by roxygen2: do not edit by hand

export(HospiNet)
export(all_admissions_summary)
export(checkBase)
export(create_fake_subjectDB)
export(create_fake_subjectDB_clustered)
export(create_subject_stay)
export(edgelist_from_base)
export(get_metrics)
export(hospinet_from_subject_database)
export(matrix_from_base)
export(matrix_from_edgelist)
export(per_facility_summary)
import(checkmate)
import(data.table)
import(ggplot2)
import(ggraph)
import(igraph)
importFrom(R6,R6Class)
importFrom(stats,rnorm)
