context("app-function")
# This file is for testing the applications in the app/ directory.

library(shinytest)

test_that("shiny_app() works", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()
  
  expect_pass(testApp("./apps/", compareImages = FALSE))
})